@@ -96,14 +96,16 @@ class BaseShaper:
     def position(self):
         """Run the positioning stage."""
         self._run_stage("pos")
+        self.buffer.clear_mask()
         # zero width marks
-        # for i in self.buffer.items:
-        #     if i.category[0] == "mark":
-        #         i.position.xAdvance = 0
+        for i in self.buffer.items:
+            if i.category[0] == "mark":
+                i.position.xAdvance = 0
         # zero width default ignorables
         self.zero_width_default_ignorables()
         for i in range(0,len(self.buffer.items)):
             self.propagate_attachment_offsets(i)
+        self.plan.msg("Positioning done", self.buffer)
 
     def propagate_attachment_offsets(self, i):
         """Resolve attachment offsets for item ``i``."""
@@ -119,7 +121,8 @@ class BaseShaper:
             return
         self.propagate_attachment_offsets(j)
         if attach_type == "cursive":
-            self.buffer.items[i].position.yPlacement = (self.buffer.items[i].position.yPlacement or 0) + (self.buffer.items[j].position.yPlacement or 0) # XXX Horizontal only
+            pass
+            # self.buffer.items[i].position.yPlacement = (self.buffer.items[i].position.yPlacement or 0) + (self.buffer.items[j].position.yPlacement or 0) # XXX Horizontal only
         else:
             self.buffer.items[i].position.xPlacement += self.buffer.items[j].position.xPlacement or 0
             self.buffer.items[i].position.yPlacement += self.buffer.items[j].position.yPlacement or 0
